
/*  
 *  *********************************************
 *  * This Sketch is written by Matthias Lorenz * 
 *  *********************************************
 *
 *  You are using it at your own risk!
 *  
 *  
 *  
 *  Tested with Teensy LC + Arduino IDE 1.6.6
 *  
 *  
 *  
 *  
 *   
 *  Hint:
 *  ----- 
 *  
 * 
 *  I've read a lot about not working SDCards with this Module, but I can say 
 *  in 99% of all cases it's not the SDCard, it's how the SDCard is formatted.
 *  
 *  I'm a Mac User and I've used the "Disk Utillity", but this is not a good idea,
 *  because often I can't use it with this module.
 *  
 *  The solution is the "SDFormatter" that is available for Mac OSX and Windows.
 *  I've tested both (under OSX and Windows) and after that, the samples played 
 *  fine without any issues.
 *  
 *  
 *  Tested SDCards:
 *  
 *  Transcend 512 MB: Working
 *  Transcend 1 GB: Working
 *  Transcend 2 GB: Working
 *  Transcend 4 GB: NOT Working
 *  Transcend 8 GB: NOT Working
 * 
 * 
 *  My Modules are working fine with 3.3V. I think it's dependend of the used SDCard.
 *  I've read that someone is using 3.5-3,7V to make it working, also I've read from
 *  5V soldered bridges ... but I can not confirm any differences against 3.3V. So I 
 *  would use 3.3V everytime, because 5V on the Pins can destroy the WTV020 chip.
 * 
 * 
 *  Often the WTV020 Arduino Library was not working... that's the reason why I not 
 *  using it. I've tested a lot with the timing ranges, and I think this sketch is 
 *  mostly perfect in all situations. 
 *  
 *  
 *  Volume Control (0-7) is buggy in the WTV020 Module: the value can not be lower 
 *  than 4, volume value < 4 are producing random volume modulations => distrotion. 
 *  And also you can not raipidly jumping to any volume values (also volume modulations 
 *  => distrotion). This is the reason for my modified module stepped volume control. 
 *  You can set the volume from 0 (off) to 4. If you are jumping from 4 to 1, then the 
 *  sketch will lower the volume value every 800ms in 3 steps (4...3...2...1). This 
 *  is stable and tested with three WTV020 modules.
 * 
 * 
 *  I was searching for a small sound card solution with Arduino, so I've piggybacked
 *  the WTV020 module on a Teensy LC. I've soldered the complete WTV020 module directly
 *  at the Teensy LC (but it also works with small Arduino boards):
    
   
                                        USB
                                      ┌────┐
                                 -----┘    └-----
                          GND   o                o   +5V
                            0   o                o   
                            1   o     Teensy     o   + 3.3V
                            2   o       LC       o   23
                            3   o                o   22
                            4   o                o   21
                            5   o                o   20 
        Teensy Pins              -------  -------                   Teensy Pins
            6          /RESET   o       \/       o   +3.3V              19
            7           AUDIO   o     WTV020     o   /BUSY              18
            8              NC   x    --------    x   NC                 17
            9            SPK+   x   | |||||| |   x   Next Sound         16
           10            SPK-   x   |        |   x   Prev Sound         15
           11              NC   x   |   SD   |   x   NC                 14
           12             CLK   o   |        |   o   DATA               13
                                 --            --
                          GND   o    --------    o   Pause/Play

                                 
  (x = removed male pins from the WTV020 module, to have more pins free to use for servos)



  If you need a little bit more and precise volume control, then you can use a 
  Digital Potentiometer and control it with an Analog Pin.


  
  To convert files into AD4 files easly, I've written my own tool in Java. This tool
  will tune the source file automaticaly (Mono, 32 Khz, -6 dB) and it can convert wav, 
  ogg, mp3 files into ad4 files. Also you can convert ad4 files into wav files.

  But all source files must be in 16 bit (this tool can not convert 8 Bit or 24 Bit files!) 



  File-Format:
  -------------
    
  AD4 Files: 16 Bit, 6KHz to 36KHz (4 Bit ADCPM format files), -6db
  
  0000.ad4
  0001.ad4
  .
  0XXX.ad4 (max. 512 files)

                
*/



boolean debug = true;


const int resetPin  = 6; 
const int clockPin  = 12;
const int dataPin   = 13;
const int busyPin   = 18;

int busyPinState = HIGH;



int volume = 4;

void setup() {

  if(debug) Serial.begin(115200);
   
  pinMode(resetPin, OUTPUT);
  pinMode(clockPin, OUTPUT);
  pinMode(dataPin, OUTPUT);
  pinMode(busyPin, INPUT);


  pinMode(A1, OUTPUT); 
  

  delay(1000); // * Wait for the Pins *
  
  resetModule();
  
}

void loop() {

  // * A short testcase: *

  startSoundInBackground(0);


  setVolume(volume);
  delay(20000);
  
  /*
  setVolume(1);
  delay(3000);
  setVolume(3);
  delay(3000);
  setVolume(1);
  delay(3000);
  setVolume(0);
  delay(3000); 
  setVolume(4);
  delay(3000); 
  setVolume(2);
  delay(3000); 
  setVolume(3);
  delay(3000);
  setVolume(4);
  delay(3000);
*/
  
  delay(2000);
    
}


void resetModule(){
  
  digitalWrite(resetPin, HIGH);  
  digitalWrite(resetPin, LOW);
  delay(5);
  
  digitalWrite(resetPin, HIGH);
  digitalWrite(clockPin,HIGH);

  // * Wait *
  delay(700);

  // * Volume has to be "FULL VOLUME" 7  ... must be set, because after Power on or Reset Button sometimes the sound will not start! *
  sendCommand(0xfff7);  
}


void startSoundInBackground(int soundNumber){
  
  sendCommand(soundNumber);  
}

void startSoundAndWaitTilEnd(int soundNumber){
  
  sendCommand(soundNumber);
  
  busyPinState=digitalRead(busyPin);
  while(busyPinState==HIGH) busyPinState=digitalRead(busyPin);  
}

void stopSound() {  
  sendCommand(0xFFFF);  
}


// * Volume 0-4 (0 = Off) *
void setVolume(int targetVol) {

  // * the value can not be lower 4, volume < 4 = random volume modulation => distrotion *
  if(targetVol >=0 && targetVol < 5) { 

    if(targetVol == 0)   sendCommand(0xFFF0);
    else {

      if(targetVol > volume) {

        int steps = targetVol-volume;
        for(int i=0;i<steps;i++) {
          volume++;
          sendCommand(0xFFF0+3+volume); 
          delay(800); // * without this delay... the sound will destroyed *
        }        
      }
      else if(targetVol < volume) {

        int steps = volume-targetVol;        
        for(int i=0;i<steps;i++) {
          volume--;
          sendCommand(0xFFF0+3+volume); 
          delay(800); // * without this delay... the sound will destroyed *
        }
      }
    }
  }
}


void sendCommand(unsigned int command) {

  if(debug) Serial.println("--------------");
  if(debug) Serial.print("Command (");
  if(debug) Serial.print(command, HEX);
  if(debug) Serial.println(")");
  
  //Start bit Low level pulse.
  digitalWrite(clockPin, LOW);
  delay(2);


  // * 0x8000 = 100000000 00000000 = 16 Bits, Send Command as Serial Transfer *
  for (unsigned int mask = 0x8000; mask > 0; mask >>= 1) {
    
    digitalWrite(clockPin, LOW);
    delayMicroseconds(50);
    
    // * Set Data-Bit on the Line *
    if (command & mask) {
      
      Serial.print("1");
      digitalWrite(dataPin, HIGH);    
    }
    else  {
      
      if(debug) Serial.print("0");
      digitalWrite(dataPin, LOW);
    }
    if(debug) Serial.print(" ");
  
    // * Wait 50us *
    delayMicroseconds(50); 
    
    // * Clock: High ... time to get Data-Bit *
    digitalWrite(clockPin, HIGH);

    // * Wait some time to give time to read the bit (possible values: 5-6500us) *
    delayMicroseconds(50);
  }

  if(debug) Serial.println(" ");
 

  // * Give Busy time to go "High" *
  delay(50);
}


